<?php

defined('ABSPATH') or die('No direct script access allowed.');


class ExpayProvider
{
    private $url;
    private $public_key;
    private $private_key;
    private $testmode;
    private $curl;

    public function __construct()
    {
        if (!$this->public_key = WC()->session->get('publishable_key', '')) {
            error_log('Publishable key of marketplace not set');
        }

        if (!$this->private_key = WC()->session->get('private_key', '')) {
            error_log('Private key of marketplace not set');
        }

        $this->testmode = WC()->session->get('testmode', '');

        if (!$this->url = $this->testmode ? WC()->session->get('sandbox_url', '') : WC()->session->get('live_url', '')) {
            error_log('Sandbox url or Live url of marketplace not set');
        }

        $this->curl = new Curl();
    }

    protected function buildUrl(string $route): string
    {
        return $this->url . '/' . $route;
    }

    protected function post(string $action, $data = []): array
    {
        $data['__METHOD__'] = $action;
        $response = $this->curl->post($this->buildUrl($action), $this->prepareData($data));

        if ($this->testmode) {
            if (!empty($response['body']['response'])) {
                error_log('Result for ' . $this->buildUrl($action) . ': ' . is_array($response['body']['response']) ? json_encode($response['body']['response']) : json_encode($response));
            } else {
                error_log('Result for ' . $this->buildUrl($action) . ': ' . is_array($response['body']) ? json_encode($response['body']) : json_encode($response));
            }
        }

        if (isset($response['curl_error']) && $response['curl_error'] != '') {
            error_log('Result for ' . $this->buildUrl($action) . ': ' . 'Service is unavailable');
        }

        if (isset($response['body']['error'])) {
            error_log('Result for ' . $this->buildUrl($action) . ': ' . is_array($response['body']['error']) ? json_encode($response['body']['error']) : json_encode($response));
        }

        $result = $response['body']['response'] ?? [];
        if (empty($result)) {
            $error = $response['curl_error'] != '' ? $response['curl_error'] : $response['body']['error']['message'] ?? $response['body'];
            $result = ['error' => $error, 'code' => $response['body']['error']['code'] ?? 500];
        }

        if (!empty($result['error']) && function_exists('wc_add_notice') && function_exists('wc_clear_notices')) {
            wc_clear_notices();
            wc_add_notice($result['error'], 'error');
        }
        return $result;
    }

    public function prepareData(array $data)
    {
        $method = $data['__METHOD__'];
        unset($data['__METHOD__']);
        $data['key'] = $this->public_key;
        $data['timestamp'] = time();
        $data['hash'] = $this->hashRequest($data, $method . '?');
        return $data;
    }

    public function hashRequest($params, $method = null)
    {
        $string_for_hash = null;

        if ($method) {
            $string_for_hash = $method . http_build_query($params, null, '&', PHP_QUERY_RFC3986);
        } else {
            $string_for_hash = http_build_query($params, null, '&', PHP_QUERY_RFC3986);
        }

        return hash_hmac('sha1', $string_for_hash, $this->private_key);
    }

    public function hashResponse($response)
    {
        return hash_hmac('sha1', json_encode($response), $this->private_key);
    }

    public function getMethodList()
    {
        $response = $this->post('getMethods');
        $method_list = $response['methods'] ?? [];
        if (is_array($method_list)) {
            foreach ($method_list as $method) {
                $result[$method['id']] = $method;
            }
        }
        return $result ?? [];
    }

    public function getServicesCategories()
    {
        $response = $this->post('getCategories');
        $categories = $response['categories'] ?? [];
        foreach ($categories as $category) {
            $result[$category['key']] = $category;
        }

        return $result ?? [];
    }

    public function initPayment($service_id, $amount, $order, $url_success, $url_waiting, $url_reject, $attrs = [])
    {
        $data = [
            'service_id' => $service_id,
            'amount' => $amount,
            'order' => $order,
            '_successUrl' => $url_success,
            '_waitingUrl' => $url_waiting,
            '_rejectUrl' => $url_reject,
        ];

        foreach ($attrs as $key => $value) {
            $data[$key] = $value;
        }

        return $this->post('initPayment', $data);
    }

    public function getPaymentStatus($payment_id)
    {
        $data = ['payment_id' => $payment_id];
        $response = $this->post('getStatus', $data);
        return $response['status'] ?? [];
    }

    public function sendResponse($code, $message)
    {
        $response = [
            'response' => [
                'status' => $code,
                'message' => $message,
                'timestamp' => time()
            ]
        ];
        $response['hash'] = $this->hashResponse($response['response']);
        header('Content-type: application/json');
        exit(json_encode($response));
    }
}